#!/usr/bin/python

################################################################
#                                                              #
# Maligno clientgen                                            #
# written by Juan J. Guelfo @ Encripto AS                      #
# post@encripto.no                                             #
#                                                              #
# Copyright 2013-2014 Encripto AS. All rights reserved.        #
#                                                              #
# Maligno is licensed under the FreeBSD license.               #
# http://www.freebsd.org/copyright/freebsd-license.html        #
#                                                              #
################################################################



import re, os, getopt, ConfigParser, sys

__version__ = "1.0"
__author__ = "Juan J. Guelfo, Encripto AS (post@encripto.no)"


# Prints title and other header info
def header():
    os.system("clear")
    
    print ""
    print " ================================================================= "
    print "|  Maligno clientgen v{0}: Maligno client generator \t\t  |".format(__version__)
    print "|  by {0}\t\t  |".format(__author__)
    print " ================================================================= "
    print ""

    
# Prints help    
def help():
    header()
    print """
   Usage: python clientgen.py [mandatory options]

   Mandatory options:
       -i  num          ...Payload index to request from Maligno
       -f  file         ...Maligno server config file
       -o  file         ...Output file
        
   Example:
       python clientgen.py -i 0 -f server.conf -o my_client.py
    """
    sys.exit(0) 

    
# Imports server configuration and does a few basic checks
def get_server_config(config_file):
    config_variables = [
        "base64rounds", "blocksize", "paddingchar", "secret", 
        "srvaddr", "srvport", "ssl", "sslcert", "banner",
        "reqparam", "useragent"
        ]

    config = ConfigParser.ConfigParser()
    config.readfp(open(config_file))
    
    server_config = config.items("Encryption")
    server_config += config.items("Server")
            
    for e in server_config:
        if e[0] not in config_variables:
            print "\n[-] Server config variable duplicated or not recognized: {0}\n".format(e[0])
            sys.exit()
        
        if not e[1]:
            print "\n[-] Server config variable is empty: {0}\n".format(e[0])
            sys.exit()
        
        config_variables.remove(e[0])
    
    if config_variables:
        print "\n[-] Server config variables are missing: {0}".format(str(config_variables).strip("[]").replace("'", ""))
        sys.exit()
    
    return dict(x[0:] for x in reversed(server_config))


def get_client_template(template_file):
    try:
        fr = open(template_file, 'r')
        content = fr.read().splitlines()
        fr.close()
    
    except IOError:
        print "[-] Could not find {0}.".format(template_file)
        sys.exit()
    
    return content
 
 
def generate_client_code(server_config, client_template, payload_index):
    client_code = []
    for template_line in client_template:
        if template_line == "<ENCODINGROUNDS>":
            code_line = "encoding_rounds = " + server_config["base64rounds"]
        
        elif template_line == "<SECRET>":
            code_line = "secret = \"" + server_config["secret"] + "\""
        
        elif template_line == "<REQPARAM>":
            code_line = "req_param = \"" + server_config["reqparam"] + "\""
        
        elif template_line == "<INDEX>":
            code_line = "index = " + str(payload_index)
        
        elif template_line == "<SSL>":
            if server_config["ssl"] == "true" or server_config["ssl"] == "True":
                code_line = "ssl = True"
            else:
                code_line = "ssl = False"
                
        elif template_line == "<SRVADDR>":
            code_line = "server_addr = \"" + server_config["srvaddr"] + "\""
        
        elif template_line == "<SRVPORT>":
            code_line = "server_port = \"" + server_config["srvport"] + "\""
        
        elif template_line == "<HEADERS>":
            code_line = "headers = { \"User-Agent\" : \"" + server_config["useragent"] + "\" }"
        
        elif template_line.startswith("#"):
            continue
        
        else:
            code_line = template_line
            
        client_code.append(code_line)
    
    return client_code

    
    
def generate_client_file(client_code, output_file):
    try:
        fw = open(output_file, 'w')
        fw.write("#!/usr/bin/python\n")
        
        for line in client_code:
            fw.write(line + "\n")
            
        fw.close()
        
    except IOError:
        print "[-] Could not save client code in {0}.".format(output_file)
    
    return
    

if __name__ == '__main__':
    
        # Parse options
    try:
        options, args = getopt.getopt(sys.argv[1:], "i:f:o:", ["index=", "file=", "output="])

    except getopt.GetoptError, err:
        header()
        print "\n[-] ERROR: {0}.\n".format(str(err))
        sys.exit(1)
    
    if not options:
        help()

    payload_index = None
    config_file = None
    output_file = None
    
    for opt, arg in options:
        if opt in ("-i"):
            payload_index = arg
        
        if opt in ("-f"):
            config_file = arg    
            
        if opt in ("-o"):
            output_file = arg  
    
    if not payload_index or not config_file or not output_file:
        header()
        print "[-] ERROR: You must provide all mandatory options.\n"
        sys.exit(1)
    
    
    # Read config, generate payloads and start server
    header()

    print "[*] Reading server configuration file..."
    server_config = get_server_config(config_file)
    
    print "[*] Reading client template..."
    client_template = get_client_template("maligno_client.template")
    
    print "[*] Generating client code..."
    client_code = generate_client_code(server_config, client_template, payload_index)
    
    print "[*] Writing client code to file..."
    generate_client_file(client_code, output_file)
    
    
    print "[*] Finished!\n"
    sys.exit()